/**
 * 
 */
package com.github.ansell.oas.website.test;

import net.sourceforge.jwebunit.htmlunit.HtmlUnitTestingEngineImpl;
import net.sourceforge.jwebunit.junit.WebTester;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Rule;
import org.junit.rules.Timeout;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.gargoylesoftware.htmlunit.NicelyResynchronizingAjaxController;
import com.github.ansell.propertyutil.PropertyUtil;

/**
 * Abstracts the WebTester setup and tear down actions from test implementations so they will always
 * have a new tester and not have to deal with opening and closing issues.
 * 
 * @author Peter Ansell p_ansell@yahoo.com
 */
public abstract class AbstractOASIntegrationTest
{
    @Rule
    public Timeout timeout = new Timeout(30000);
    
    protected final Logger log = LoggerFactory.getLogger(this.getClass());
    
    private WebTester tester;
    
    protected PropertyUtil propertyUtil;
    
    protected static final String TEST_ADMIN_USER = "testAdminUser";
    
    protected static final String TEST_ADMIN_PASSWORD = "testAdminPassword";
    
    protected static final String TEST_PASSWORD = "anotherPassword";
    
    protected static final String TEST_USER = "anotherUser";
    
    /**
     * Utility method to verify that freemarker has not encountered errors when generating output.
     * 
     * @param body
     *            Generated output in which to look for possible freemarker errors.
     */
    public void assertFreemarker(final String body)
    {
        Assert.assertFalse("Freemarker error.", body.contains("Java backtrace for programmers:"));
        Assert.assertFalse("Freemarker error.", body.contains("freemarker.core."));
    }
    
    /**
     * Override this to return the http accept header for this test class.
     * 
     * @return The test HTTP Accept header to use for all requests through this class, or null to
     *         use the default Content Negotiation mechanism, which may produce random results.
     */
    protected abstract String getTestAcceptHeader();
    
    /**
     * 
     * @return The WebTester object for the current test
     */
    protected WebTester getWebTester()
    {
        return this.tester;
    }
    
    /**
     * Logs in the user with the given username and password using whatever method matches the
     * integration test implementing this method.
     * 
     * @param username
     *            The username of the user to login.
     * @param password
     *            The password of the user to login.
     */
    protected abstract void login(String username, String password);
    
    /**
     * Logs out any user that is currently logged in.
     */
    protected abstract void logout();
    
    @Before
    public void setUp() throws Exception
    {
        this.propertyUtil = new PropertyUtil("oas");
        this.setWebTester(new WebTester());
        final String portNumberString = this.propertyUtil.get("jetty.http.port", "9090");
        
        int portNumber = 9090;
        
        try
        {
            portNumber = Integer.parseInt(portNumberString);
        }
        catch(final NumberFormatException nfe)
        {
            portNumber = 9090;
        }
        
        this.log.info("Using port number: {}", portNumber);
        
        this.getWebTester().setBaseUrl("http://localhost:" + portNumber + "/oas-test/");
        this.setupTestAcceptHeader();
        
        this.getWebTester().beginAt("/");
        
        // ensure that we can perform ajax queries with little fuss
        if(this.getWebTester().getTestingEngine() instanceof HtmlUnitTestingEngineImpl)
        {
            ((HtmlUnitTestingEngineImpl)this.getWebTester().getTestingEngine()).getWebClient().setAjaxController(
                    new NicelyResynchronizingAjaxController());
        }
    }
    
    /**
     * This method is called from tests where the accept header needs to be setup.
     * 
     * HTML browser tests should set all of the normal HTML/CSS/Javascript types, while RDF tests
     * should only ever set RDF types.
     * 
     */
    protected final void setupTestAcceptHeader()
    {
        final String acceptHeader = this.getTestAcceptHeader();
        if(acceptHeader != null)
        {
            this.getWebTester().getTestContext().addRequestHeader("Accept", acceptHeader);
        }
    }
    
    /**
     * 
     * @param tester
     *            The new WebTester instance for the next test about to be executed
     */
    private void setWebTester(final WebTester tester)
    {
        this.tester = tester;
    }
    
    /**
     * @throws java.lang.Exception
     */
    @After
    public void tearDown() throws Exception
    {
        try
        {
            this.logout();
        }
        catch(final Exception ex)
        {
            this.log.error("Found exception in logout after test", ex);
        }
        
        try
        {
            // Reset using the maven build configured reset key from oas.properties
            this.getWebTester().gotoPage("/reset/" + this.propertyUtil.get("oas.test.website.reset.key", ""));
        }
        catch(final Exception ex)
        {
            this.log.error("Found exception resetting application after test", ex);
        }
        
        try
        {
            this.getWebTester().closeBrowser();
        }
        catch(final Exception ex)
        {
            this.log.error("Found exception closing browser after test", ex);
        }
        
        this.setWebTester(null);
    }
    
}
