/**
 * 
 */
package com.github.ansell.oas.website.test;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URISyntaxException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;
import java.nio.file.StandardOpenOption;

import net.sourceforge.jwebunit.api.IElement;

import org.apache.commons.io.IOUtils;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Ignore;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.gargoylesoftware.htmlunit.FailingHttpStatusCodeException;
import com.github.ansell.oas.test.TestUtils;

/**
 * Tests the OAS Services with application/json requests, which should mimic those from browsers.
 * Other tests can be setup to allow other Accept headers.
 * 
 * @author Peter Ansell p_ansell@yahoo.com
 */
public class OASServicesJsonIntegrationTest extends AbstractOASIntegrationTest
{
    private static final Logger LOGGER = LoggerFactory.getLogger(OASServicesJsonIntegrationTest.class);
    
    @Rule
    public TemporaryFolder folder = new TemporaryFolder();
    
    private File testDataFolder;
    
    @Override
    protected String getTestAcceptHeader()
    {
        return "application/json";
    }
    
    @Override
    protected void login(final String username, final String password)
    {
        // NOTE: This only works currently because Restlet responds to our "application/json"
        // request with a "text/html" because there is no better match.
        // If that ever changes we may need to change @Get("html") in LoginResource.getLoginPageHtml
        // to @Get
        this.getWebTester().beginAt("/login");
        
        final IElement userElement = this.getWebTester().getElementById("oaslogin");
        userElement.setAttribute("value", username);
        
        final IElement passwordElement = this.getWebTester().getElementById("oaspassword");
        passwordElement.setAttribute("value", password);
        
        this.getWebTester().clickButtonWithText("Login");
    }
    
    @Override
    protected void logout()
    {
        this.getWebTester().gotoPage("/dologout");
    }
    
    @Override
    @Before
    public void setUp() throws Exception
    {
        super.setUp();
        
        this.testDataFolder = this.folder.newFolder();
    }
    
    @Ignore
    @Test
    public void testCreateOntologyPostRdfWebService() throws IOException
    {
        this.login("anotherUser", "anotherPassword");
        
        // begin at the test html page that contains a form to submit the data to the webservice
        this.getWebTester().gotoPage("/resources/static/oas-test.html");
        
        Path temp = null;
        try
        {
            // Create temp file using the new Java-7 NIO API
            temp = Files.createTempFile(Paths.get(this.testDataFolder.toURI()), "testontologydata-", ".rdf");
            
            Assert.assertTrue(Files.isWritable(temp));
            
            try (OutputStream outputStream = Files.newOutputStream(temp, StandardOpenOption.WRITE))
            {
                IOUtils.write(TestUtils.createTestOntologyDocument("application/rdf+xml",
                        "http://example.org/ontology/me/", "http://versions.other.example.org/ontology/me/1"),
                        outputStream);
            }
        }
        catch(final IOException e)
        {
            Assert.fail(e.toString());
        }
        
        final String filename = temp.toAbsolutePath().toString();
        OASServicesJsonIntegrationTest.LOGGER.info("filename={}", filename);
        this.getWebTester().setWorkingForm("testontologycreateform");
        this.getWebTester().setTextField("ontologyFile", filename);
        
        // submit the page to /oas-test/services/testontologies/ontologymanager/upload where it
        // should be handled using the multipart/form-data request content type handler
        this.getWebTester().submit("submit");
        
        Files.deleteIfExists(temp);
        
        Assert.assertTrue(this.getWebTester().getTestingEngine().getPageURL().toExternalForm()
                .endsWith("/ontologymanager/"));
        
        this.getWebTester().assertHeaderEquals("Content-Type", "application/rdf+json; charset=UTF-8");
        
        this.getWebTester().assertResponseCode(200);
        
    }
    
    @Ignore
    @Test
    public void testCreateOntologyPostRdfWebServiceMediumSize() throws IOException, URISyntaxException
    {
        this.login("anotherUser", "anotherPassword");
        
        // begin at the test html page that contains a form to submit the data to the webservice
        this.getWebTester().gotoPage("/resources/static/oas-test.html");
        
        Path temp = null;
        try
        {
            // Create temp file using the new Java-7 NIO API
            temp = Files.createTempFile(Paths.get(this.testDataFolder.toURI()), "testontologydata-", ".rdf");
            Assert.assertTrue(Files.isWritable(temp));
            
            // copy the resource to the temporary file so it can be uploaded, as the resource may be
            // inside a jar file and not-locatable by the web browsing test engine
            // Use Java-7 try-with-resource block
            try (InputStream input = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl"))
            {
                final long copy = Files.copy(input, temp, StandardCopyOption.REPLACE_EXISTING);
                
                Assert.assertTrue(copy > 0);
            }
        }
        catch(final IOException e)
        {
            Assert.fail(e.toString());
        }
        
        final String filename = temp.toAbsolutePath().toString();
        OASServicesJsonIntegrationTest.LOGGER.info("filename={}", filename);
        this.getWebTester().setWorkingForm("testontologycreateform");
        this.getWebTester().setTextField("ontologyFile", filename);
        
        // submit the page to /oas-test/services/testontologies/ontologymanager/upload where it
        // should be handled using the multipart/form-data request content type handler
        this.getWebTester().submit("submit");
        
        Files.deleteIfExists(temp);
        
        Assert.assertTrue(this.getWebTester().getTestingEngine().getPageURL().toExternalForm()
                .endsWith("/ontologymanager/"));
        
        this.getWebTester().assertHeaderEquals("Content-Type", "application/rdf+json; charset=UTF-8");
        
        this.getWebTester().assertResponseCode(200);
        
    }
    
    /**
     * Tests whether a call to the Search for Annotations by Object Type service generates an HTTP
     * 400 response.
     */
    @Test
    public void testFailEmptyAnnotationByObjectType()
    {
        try
        {
            this.getWebTester().beginAt("/annotation/byobjecttype");
            Assert.fail("Did not find expected exception");
        }
        catch(final net.sourceforge.jwebunit.exception.TestingEngineResponseException ex)
        {
            Assert.assertNotNull(ex.getCause());
            
            Assert.assertTrue(ex.getCause() instanceof FailingHttpStatusCodeException);
            
            final FailingHttpStatusCodeException fhsce = (FailingHttpStatusCodeException)ex.getCause();
            
            OASServicesJsonIntegrationTest.LOGGER.info("Found expected exception: " + fhsce.getMessage());
            
            Assert.assertEquals(400, fhsce.getStatusCode());
        }
    }
    
    /**
     * Tests whether a call to the Search for Annotations by Object URI service generates an HTTP
     * 400 response.
     */
    @Test
    public void testFailEmptyAnnotationByObjectUri()
    {
        try
        {
            this.getWebTester().beginAt("/annotation/byobjecturi");
            Assert.fail("Did not find expected exception");
        }
        catch(final net.sourceforge.jwebunit.exception.TestingEngineResponseException ex)
        {
            Assert.assertNotNull(ex.getCause());
            
            Assert.assertTrue(ex.getCause() instanceof FailingHttpStatusCodeException);
            
            final FailingHttpStatusCodeException fhsce = (FailingHttpStatusCodeException)ex.getCause();
            
            OASServicesJsonIntegrationTest.LOGGER.info("Found expected exception: " + fhsce.getMessage());
            
            Assert.assertEquals(400, fhsce.getStatusCode());
        }
    }
    
    /**
     * Tests whether a call to the Search for Annotations by Ontology Term URI service generates an
     * HTTP 400 response.
     */
    @Test
    public void testFailEmptyAnnotationByOntologyTermUri()
    {
        try
        {
            this.getWebTester().beginAt("/annotation/byontologytermuri");
            Assert.fail("Did not find expected exception");
        }
        catch(final net.sourceforge.jwebunit.exception.TestingEngineResponseException ex)
        {
            Assert.assertNotNull(ex.getCause());
            
            Assert.assertTrue(ex.getCause() instanceof FailingHttpStatusCodeException);
            
            final FailingHttpStatusCodeException fhsce = (FailingHttpStatusCodeException)ex.getCause();
            
            OASServicesJsonIntegrationTest.LOGGER.info("Found expected exception: " + fhsce.getMessage());
            
            Assert.assertEquals(400, fhsce.getStatusCode());
        }
    }
    
    /**
     * Tests whether a call to the Count Annotations by Ontology Term URI service generates an HTTP
     * 400 response.
     */
    @Test
    public void testFailEmptyAnnotationCountByOntologyTermUri()
    {
        try
        {
            this.getWebTester().beginAt("/annotation/countbyontologytermuri");
            Assert.fail("Did not find expected exception");
        }
        catch(final net.sourceforge.jwebunit.exception.TestingEngineResponseException ex)
        {
            Assert.assertNotNull(ex.getCause());
            
            Assert.assertTrue(ex.getCause() instanceof FailingHttpStatusCodeException);
            
            final FailingHttpStatusCodeException fhsce = (FailingHttpStatusCodeException)ex.getCause();
            
            OASServicesJsonIntegrationTest.LOGGER.info("Found expected exception: " + fhsce.getMessage());
            
            Assert.assertEquals(400, fhsce.getStatusCode());
        }
    }
    
    /**
     * Tests whether a call to the annotation create webservice with an empty query generates an
     * HTTP 400 error.
     */
    @Test
    public void testFailEmptyCreateAnnotation()
    {
        try
        {
            this.getWebTester().beginAt("/annotation/create");
            Assert.fail("Did not find expected exception");
        }
        catch(final net.sourceforge.jwebunit.exception.TestingEngineResponseException ex)
        {
            Assert.assertNotNull(ex.getCause());
            
            Assert.assertTrue(ex.getCause() instanceof FailingHttpStatusCodeException);
            
            final FailingHttpStatusCodeException fhsce = (FailingHttpStatusCodeException)ex.getCause();
            
            OASServicesJsonIntegrationTest.LOGGER.info("Found expected exception: " + fhsce.getMessage());
            
            Assert.assertEquals(405, fhsce.getStatusCode());
        }
    }
    
    /**
     * Tests whether a call to an unknown annotation generates a HTTP 400 response.
     */
    @Test
    public void testFailEmptyFetchUnknownAnnotation()
    {
        try
        {
            this.getWebTester().beginAt("/annotation/id/?annotationId=0123456");
            Assert.fail("Did not find expected exception");
        }
        catch(final net.sourceforge.jwebunit.exception.TestingEngineResponseException ex)
        {
            Assert.assertNotNull(ex.getCause());
            
            Assert.assertTrue(ex.getCause() instanceof FailingHttpStatusCodeException);
            
            final FailingHttpStatusCodeException fhsce = (FailingHttpStatusCodeException)ex.getCause();
            
            OASServicesJsonIntegrationTest.LOGGER.info("Found expected exception: " + fhsce.getMessage());
            
            Assert.assertEquals(400, fhsce.getStatusCode());
        }
        
    }
    
    /**
     * Tests whether an HTTP GET request to the Ontology Manager Delete service generates an HTTP
     * 405 Method Not Allowed response, as HTTP GET is not allowed by this service
     * 
     * FIXME: Restlet seems to return the response in HTML, corresponding the HTML delete form,
     * which doesn't have an equivalent description in RDF/JSON
     * 
     */
    @Ignore
    @Test
    public void testFailEmptyOntologyManagerDelete()
    {
        this.login("anotherUser", "anotherPassword");
        
        try
        {
            this.getWebTester().gotoPage("/ontologymanager/delete");
            Assert.fail("Did not find expected exception");
        }
        catch(final net.sourceforge.jwebunit.exception.TestingEngineResponseException ex)
        {
            Assert.assertNotNull(ex.getCause());
            
            Assert.assertTrue(ex.getCause() instanceof FailingHttpStatusCodeException);
            
            final FailingHttpStatusCodeException fhsce = (FailingHttpStatusCodeException)ex.getCause();
            
            OASServicesJsonIntegrationTest.LOGGER.info("Found expected exception: " + fhsce.getMessage());
            
            Assert.assertEquals(405, fhsce.getStatusCode());
        }
    }
    
    /**
     * Tests whether an HTTP GET request to the Ontology Manager Delete service generates an HTTP
     * 401 Unauthorised response without logging in.
     */
    @Test
    public void testFailEmptyOntologyManagerDeleteUnauthorised()
    {
        try
        {
            this.getWebTester().beginAt("/ontologymanager/delete");
            Assert.fail("Did not find expected exception");
        }
        catch(final net.sourceforge.jwebunit.exception.TestingEngineResponseException ex)
        {
            Assert.assertNotNull(ex.getCause());
            
            Assert.assertTrue(ex.getCause() instanceof FailingHttpStatusCodeException);
            
            final FailingHttpStatusCodeException fhsce = (FailingHttpStatusCodeException)ex.getCause();
            
            OASServicesJsonIntegrationTest.LOGGER.info("Found expected exception: " + fhsce.getMessage());
            
            Assert.assertEquals(401, fhsce.getStatusCode());
        }
    }
    
    /**
     * Tests whether an HTTP GET request to the Ontology Manager Upload service generates an HTTP
     * 405 Method Not Allowed response, as HTTP GET is not allowed by this service
     * 
     * FIXME: Restlet seems to return the response in HTML, corresponding the HTML upload form,
     * which doesn't have an equivalent description in RDF/JSON
     * 
     */
    @Ignore
    @Test
    public void testFailEmptyOntologyManagerUpload()
    {
        this.login("anotherUser", "anotherPassword");
        
        try
        {
            this.getWebTester().gotoPage("/ontologymanager/upload");
            Assert.fail("Did not find expected exception");
        }
        catch(final net.sourceforge.jwebunit.exception.TestingEngineResponseException ex)
        {
            Assert.assertNotNull(ex.getCause());
            
            Assert.assertTrue(ex.getCause() instanceof FailingHttpStatusCodeException);
            
            final FailingHttpStatusCodeException fhsce = (FailingHttpStatusCodeException)ex.getCause();
            
            OASServicesJsonIntegrationTest.LOGGER.info("Found expected exception: " + fhsce.getMessage());
            
            Assert.assertEquals(405, fhsce.getStatusCode());
        }
    }
    
    /**
     * Tests whether an HTTP GET request to the Ontology Manager Upload service generates an HTTP
     * 401 Unauthorised response if not logged in.
     */
    @Test
    public void testFailEmptyOntologyManagerUploadUnauthorised()
    {
        try
        {
            this.getWebTester().beginAt("/ontologymanager/upload");
            Assert.fail("Did not find expected exception");
        }
        catch(final net.sourceforge.jwebunit.exception.TestingEngineResponseException ex)
        {
            Assert.assertNotNull(ex.getCause());
            
            Assert.assertTrue(ex.getCause() instanceof FailingHttpStatusCodeException);
            
            final FailingHttpStatusCodeException fhsce = (FailingHttpStatusCodeException)ex.getCause();
            
            OASServicesJsonIntegrationTest.LOGGER.info("Found expected exception: " + fhsce.getMessage());
            
            Assert.assertEquals(401, fhsce.getStatusCode());
        }
    }
    
    /**
     * Tests whether an HTTP GET request to the Ontology Manager main page using application/rdf+xml
     * generates an HTTP 200 response.
     * 
     */
    @Test
    public void testSuccessEmptyOntologyManagerMainPageRdfXml()
    {
        // setupTestAcceptHeader();
        
        this.getWebTester().beginAt("/ontologymanager/");
        
        this.getWebTester().assertHeaderEquals("Content-Type", "application/rdf+json; charset=UTF-8");
        
        this.getWebTester().assertResponseCode(200);
    }
    
}
